Attribute VB_Name = "basMain"
Option Explicit

' ***************************************************************************
' Module:        basMain
'
' Description:   This application will count lines of code for both Visual
'                Basic and C projects.  It will handle VB Group projects and
'                single files from both C and VB.  In VB, a line continuation
'                character of " _" at the end of a line is not counted because
'                it is considered a continuation of the previous line.  Only
'                the first occurance of the continuation is counted.  This is
'                the main module to access Visual Basic and C source
'                code files.  This module will allow the user to select
'                either Visual Basic or C type files and determine the lines
'                of code count.
'
'                Depending of the current standard at the user's location,
'                there are three options to either add to the line count or
'                omit.  Options available are whether or not to include
'                Autogenerated code, blank lines, single braces (C only), or
'                comments in the final total.
'
'                Items NOT counted in VB are TRAILERS.  These are the logical
'                ending statements to a procedural heading.
'
'                   Example:    End Sub      End Function     End Property
'                               End Type     End If           Loop
'                               Wend         Next
'
'                Items NOT counted in C are BRACES( "{" and "}" ) on a line
'                by themselves or with a comment or a semicolon.
'
'                Be sure to make a reference to "Microsoft Script Runtime"
'                file (Scrrun.dll)
'
' ===========================================================================
'    DATE      NAME / eMAIL
'              DESCRIPTION
' -----------  --------------------------------------------------------------
' 01-NOV-2000  Kenneth Ives  kenaso@home.com
'              Module created
' ***************************************************************************

' ---------------------------------------------------------------------------
' Define type structure
' ---------------------------------------------------------------------------
  Public Type Stats_UDT
      Module_Name       As String
      TotalLines        As Long
      BlankLines        As Long
      CommentLines      As Long
      AutoGenLines      As Long
      MiscLines         As Long
      SubRoutines       As Long
      FunctionRoutines  As Long
      PropertyLet       As Long
      PropertyGet       As Long
      PropertySet       As Long
      API_Declares      As Long
      Constants         As Long
      Type_Structures   As Long
  End Type

  Private Const MAX_SIZE As Long = 260
  
' ---------------------------------------------------------------------------
' Needed to build a temporary file name
' ---------------------------------------------------------------------------
  Private Declare Function GetTempPath Lib "kernel32.dll" _
          Alias "GetTempPathA" (ByVal nBufferLength As Long, _
          ByVal lpBuffer As String) As Long
            
  Private Declare Function GetTempFileName Lib "kernel32.dll" _
          Alias "GetTempFileNameA" (ByVal lpszPath As String, _
          ByVal lpPrefixString As String, ByVal wUnique As Long, _
          ByVal lpTempFileName As String) As Long
            
Public Sub Main()

' ---------------------------------------------------------------------------
' Set up the path where all of the mail processing will take place.
' ---------------------------------------------------------------------------
  ChDrive App.Path
  ChDir App.Path
      
' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim cSetup As New clsMiscForm
  
' ---------------------------------------------------------------------------
' See if there is another instance of this program running
' ---------------------------------------------------------------------------
  If cSetup.IsAnotherInstance("count_loc") Then
      Set cSetup = Nothing
      Exit Sub
  End If
  
' ---------------------------------------------------------------------------
' Load the main form
' ---------------------------------------------------------------------------
  Set cSetup = Nothing
  Load frmMain
  
End Sub

Public Sub StopTheProgram()

' ---------------------------------------------------------------------------
' Upload all forms from memory and terminate this application
' ---------------------------------------------------------------------------
  Unload_All_Forms
  End
  
End Sub

Public Sub Unload_All_Forms()

' ---------------------------------------------------------------------------
' Unload all forms before terminating an application.  The module that calls
' this routine, usually executes "END" when it returns.
' ---------------------------------------------------------------------------

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim frm As Form
  
' ---------------------------------------------------------------------------
' For each form associated with thsi application, we will first hide the
' form, deactivate it, and then remove the form object from memory.  This
' greatly reduces the risk of leaving fragments of dead applications in
' memory.
' ---------------------------------------------------------------------------
  For Each frm In Forms
      frm.Hide
      Unload frm
      Set frm = Nothing
  Next
  
End Sub

Public Function AutoGen_Code(ByVal hFile As Integer, _
                             strRecord As String, _
                             lngBlanks As Long, _
                             blnCancel As Boolean) As Long

' ***************************************************************************
' Routine:       AutoGen_Code
'
' Description:   Counts the number of auto generated lines of code at the
'                beginning of the VB file.
'
' Parameters:    hFile - File handle of the currently open file
'                strRecord - Current record that is in the buffer
'                lngBlanks - Counter for the number of blank lines
'                       encountered
'                blnCancel = if an earlier version of VB shows up, the user
'                       can cancel processing
'
' Returns:       Number of autogenerated data lines
'
' ===========================================================================
'    DATE      NAME / eMAIL
'              DESCRIPTION
' -----------  --------------------------------------------------------------
' 01-NOV-2000  Kenneth Ives  kenaso@home.com
'              Routine created
' ***************************************************************************

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim intResp              As Integer
  Dim lngCount             As Long
  Dim strTmp               As String
  Dim blnFound1stOccurance As Boolean
  
  Const VB_ATTRIB          As String = "Attribute "
  
' ---------------------------------------------------------------------------
' Initialize variables
' ---------------------------------------------------------------------------
  lngCount = 0
  lngBlanks = 0
  
' ---------------------------------------------------------------------------
' Test the first record
' ---------------------------------------------------------------------------
  If StrComp(Left(strRecord, 10), "version 2.", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 10), "version 3.", vbTextCompare) = 0 Then
         intResp = MsgBox("Cannot accurately process project created " & _
                   vbCrLf & " with Visual Basic 3.0 or below", _
                   vbInformation + vbOKCancel, "Out of date")
         
         If intResp = vbCancel Then
             blnCancel = True
         End If
         GoTo Normal_Exit
         
  ' begins with single quote or blank line
  ' then leave
  ElseIf Left(strRecord, 1) = "'" Or _
         Len(strRecord) = 0 Then
      GoTo Normal_Exit
      
  ElseIf Len(strRecord) > 0 Then
      lngCount = 1  ' Assume Auto generated data
  End If
  
' ---------------------------------------------------------------------------
' See if the first line begins with "Attribute "
' ---------------------------------------------------------------------------
  strTmp = Left(strRecord, 10)
  If StrComp(strTmp, VB_ATTRIB, vbTextCompare) = 0 Then
      ' we found the first occurance
      blnFound1stOccurance = True
  Else
      blnFound1stOccurance = False
  End If
  
' ---------------------------------------------------------------------------
' Loop thru the file until we get to the end of the autogenerated code
' ---------------------------------------------------------------------------
  Do
      ' read a record
      Line Input #hFile, strRecord
      
      ' remove any leading and trailing spaces
      strRecord = CleanUpData(strRecord)
      strTmp = Left(strRecord, 10)
      
      ' see if we have found the first of occurance of "Attribute VB_"
      If blnFound1stOccurance Then
          If StrComp(strTmp, VB_ATTRIB, vbTextCompare) <> 0 Then
              ' We have now passed the autogenerated code
              Exit Do
          End If
      Else
          If StrComp(strTmp, VB_ATTRIB, vbTextCompare) = 0 Then
              ' we found the first occurance
              blnFound1stOccurance = True
          End If
      End If
          
      If Len(strRecord) > 0 Then
          lngCount = lngCount + 1   ' Auto generated data
      Else
          lngBlanks = lngBlanks + 1 ' Blank record
      End If
      
  Loop While Not EOF(hFile)
  
Normal_Exit:
' ---------------------------------------------------------------------------
' Return the accumulated value
' ---------------------------------------------------------------------------
  AutoGen_Code = lngCount
  
End Function
Public Function SubRoutine(ByVal strRecord As String, _
                           blnLineContinued As Boolean) As Boolean

' ***************************************************************************
' Routine:       SubRoutine
'
' Description:   Parses a record to see if it is a Sub routine in the VB file
'
' Parameters:    strRecord - data string to parse
'                blnLineContinued - TRUE or FALSE if line has a continuation
'                                   character at the end ( _)
'
' Returns:       TRUE or FALSE based on findings
'
' ===========================================================================
'    DATE      NAME / eMAIL
'              DESCRIPTION
' -----------  --------------------------------------------------------------
' 01-NOV-2000  Kenneth Ives  kenaso@home.com
'              Routine created
' ***************************************************************************

' ---------------------------------------------------------------------------
' Test the data string
' ---------------------------------------------------------------------------
  If StrComp(Left(strRecord, 4), "Sub ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 12), "Private Sub ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 11), "Friend Sub ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 11), "Public Sub ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 11), "Global Sub ", vbTextCompare) = 0 Then
     
      ' see if the line is continued
      If Right(strRecord, 2) = " _" Then
          blnLineContinued = True
      End If
         
      SubRoutine = True
  Else
      SubRoutine = False
  End If
 
End Function
Public Function FunctionRoutine(ByVal strRecord As String, _
                                blnLineContinued As Boolean) As Boolean

' ***************************************************************************
' Routine:       FunctionRoutine
'
' Description:   Parses a record to see if it is a Function routine in the
'                VB file
'
' Parameters:    strRecord - data string to parse
'                blnLineContinued - TRUE or FALSE if line has a continuation
'                                   character at the end ( _)
'
' Returns:       TRUE or FALSE based on findings
'
' ===========================================================================
'    DATE      NAME / eMAIL
'              DESCRIPTION
' -----------  --------------------------------------------------------------
' 01-NOV-2000  Kenneth Ives  kenaso@home.com
'              Routine created
' ***************************************************************************

' ---------------------------------------------------------------------------
' Test the data string
' ---------------------------------------------------------------------------
  If StrComp(Left(strRecord, 9), "Function ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 17), "Private Function ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 16), "Friend Function ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 16), "Public Function ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 16), "Global Function ", vbTextCompare) = 0 Then
     
      ' see if the line is continued
      If Right(strRecord, 2) = " _" Then
          blnLineContinued = True
      End If
         
      FunctionRoutine = True
  Else
      FunctionRoutine = False
  End If
 
End Function
Public Function API_Declare(ByVal strRecord As String, _
                            blnLineContinued As Boolean) As Boolean

' ***************************************************************************
' Routine:       API_Declare
'
' Description:   Parses a record to see if it is an API declare in the
'                VB file
'
' Parameters:    strRecord - data string to parse
'                blnLineContinued - TRUE or FALSE if line has a continuation
'                                   character at the end ( _)
'
' Returns:       TRUE or FALSE based on findings
'
' ===========================================================================
'    DATE      NAME / eMAIL
'              DESCRIPTION
' -----------  --------------------------------------------------------------
' 01-NOV-2000  Kenneth Ives  kenaso@home.com
'              Routine created
' ***************************************************************************

' ---------------------------------------------------------------------------
' Test the data string
' ---------------------------------------------------------------------------
  If StrComp(Left(strRecord, 8), "Declare ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 16), "Private Declare ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 15), "Public Declare ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 15), "Global Declare ", vbTextCompare) = 0 Then
     
      ' see if the line is continued
      If Right(strRecord, 2) = " _" Then
          blnLineContinued = True
      End If
         
      API_Declare = True
  Else
      API_Declare = False
  End If
 
End Function
Public Function ConstantVar(ByVal strRecord As String, _
                            blnLineContinued As Boolean) As Boolean

' ***************************************************************************
' Routine:       ConstantVar
'
' Description:   Parses a record to see if it is a constant variable in the
'                VB file
'
' Parameters:    strRecord - data string to parse
'                blnLineContinued - TRUE or FALSE if line has a continuation
'                                   character at the end ( _)
'
' Returns:       TRUE or FALSE based on findings
'
' ===========================================================================
'    DATE      NAME / eMAIL
'              DESCRIPTION
' -----------  --------------------------------------------------------------
' 01-NOV-2000  Kenneth Ives  kenaso@home.com
'              Routine created
' ***************************************************************************

' ---------------------------------------------------------------------------
' Test the data string
' ---------------------------------------------------------------------------
  If StrComp(Left(strRecord, 6), "Const ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 14), "Private Const ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 13), "Public Const ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 13), "Global Const ", vbTextCompare) = 0 Then
         
      ' see if the line is continued
      If Right(strRecord, 2) = " _" Then
          blnLineContinued = True
      End If
  
      ConstantVar = True
  Else
      ConstantVar = False
  End If
 
End Function
Public Function Type_Structure(ByVal strRecord As String, _
                               blnLineContinued As Boolean) As Boolean

' ***************************************************************************
' Routine:       Type_Structure
'
' Description:   Parses a record to see if it is a type structure in the
'                VB file
'
' Parameters:    strRecord - data string to parse
'                blnLineContinued - TRUE or FALSE if line has a continuation
'                                   character at the end ( _)
'
' Returns:       TRUE or FALSE based on findings
'
' ===========================================================================
'    DATE      NAME / eMAIL
'              DESCRIPTION
' -----------  --------------------------------------------------------------
' 01-NOV-2000  Kenneth Ives  kenaso@home.com
'              Routine created
' ***************************************************************************

' ---------------------------------------------------------------------------
' Test the data string
' ---------------------------------------------------------------------------
  If StrComp(Left(strRecord, 5), "Type ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 13), "Private Type ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 12), "Public Type ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 12), "Global Type ", vbTextCompare) = 0 Then
         
      ' see if the line is continued
      If Right(strRecord, 2) = " _" Then
          blnLineContinued = True
      End If
      
      Type_Structure = True
  Else
      Type_Structure = False
  End If
 
End Function
Public Function Comments(ByVal strRecord As String, _
                         blnLineContinued As Boolean) As Boolean

' ***************************************************************************
' Routine:       Comments
'
' Description:   Parses a record to see if it is a comment line in the
'                VB file
'
' Parameters:    strRecord - data string to parse
'                blnLineContinued - TRUE or FALSE if line has a continuation
'                                   character at the end ( _)
'
' Returns:       TRUE or FALSE based on findings
'
' ===========================================================================
'    DATE      NAME / eMAIL
'              DESCRIPTION
' -----------  --------------------------------------------------------------
' 01-NOV-2000  Kenneth Ives  kenaso@home.com
'              Routine created
' ***************************************************************************

' ---------------------------------------------------------------------------
' Test the data string
' ---------------------------------------------------------------------------
  If StrComp(Left(strRecord, 1), "'", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 4), "Rem ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 10), "Attribute ", vbTextCompare) = 0 Then
         
      ' see if the line is continued
      If Right(strRecord, 2) = " _" Then
          blnLineContinued = True
      End If
      
      Comments = True
  Else
      Comments = False
  End If
 
End Function
Public Function PropertyLet(ByVal strRecord As String, _
                            blnLineContinued As Boolean) As Boolean

' ***************************************************************************
' Routine:       PropertyLet
'
' Description:   Parses a record to see if it is a Property Let routine in
'                the VB file
'
' Parameters:    strRecord - data string to parse
'                blnLineContinued - TRUE or FALSE if line has a continuation
'                                   character at the end ( _)
'
' Returns:       TRUE or FALSE based on findings
'
' ===========================================================================
'    DATE      NAME / eMAIL
'              DESCRIPTION
' -----------  --------------------------------------------------------------
' 01-NOV-2000  Kenneth Ives  kenaso@home.com
'              Routine created
' ***************************************************************************

' ---------------------------------------------------------------------------
' Test the data string
' ---------------------------------------------------------------------------
  If StrComp(Left(strRecord, 13), "Property Let ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 21), "Private Property Let ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 20), "Friend Property Let ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 20), "Public Property Let ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 20), "Global Property Let ", vbTextCompare) = 0 Then
         
      ' see if the line is continued
      If Right(strRecord, 2) = " _" Then
          blnLineContinued = True
      End If
      
      PropertyLet = True
  Else
      PropertyLet = False
  End If
 
End Function

Public Function PropertyGet(ByVal strRecord As String, _
                            blnLineContinued As Boolean) As Boolean

' ***************************************************************************
' Routine:       PropertyGet
'
' Description:   Parses a record to see if it is a Property Get routine in
'                the VB file
'
' Parameters:    strRecord - data string to parse
'                blnLineContinued - TRUE or FALSE if line has a continuation
'                                   character at the end ( _)
'
' Returns:       TRUE or FALSE based on findings
'
' ===========================================================================
'    DATE      NAME / eMAIL
'              DESCRIPTION
' -----------  --------------------------------------------------------------
' 01-NOV-2000  Kenneth Ives  kenaso@home.com
'              Routine created
' ***************************************************************************

' ---------------------------------------------------------------------------
' Test the data string
' ---------------------------------------------------------------------------
  If StrComp(Left(strRecord, 13), "Property Get ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 21), "Private Property Get ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 20), "Friend Property Get ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 20), "Public Property Get ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 20), "Global Property Get ", vbTextCompare) = 0 Then
         
      ' see if the line is continued
      If Right(strRecord, 2) = " _" Then
          blnLineContinued = True
      End If
      
      PropertyGet = True
  Else
      PropertyGet = False
  End If
 
End Function

Public Function PropertySet(ByVal strRecord As String, _
                            blnLineContinued As Boolean) As Boolean

' ***************************************************************************
' Routine:       PropertySet
'
' Description:   Parses a record to see if it is a Property Set routine in
'                the VB file
'
' Parameters:    strRecord - data string to parse
'                blnLineContinued - TRUE or FALSE if line has a continuation
'                                   character at the end ( _)
'
' Returns:       TRUE or FALSE based on findings
'
' ===========================================================================
'    DATE      NAME / eMAIL
'              DESCRIPTION
' -----------  --------------------------------------------------------------
' 01-NOV-2000  Kenneth Ives  kenaso@home.com
'              Routine created
' ***************************************************************************

' ---------------------------------------------------------------------------
' Test the data string
' ---------------------------------------------------------------------------
  If StrComp(Left(strRecord, 13), "Property Set ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 21), "Private Property Set ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 20), "Friend Property Set ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 20), "Public Property Set ", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 20), "Global Property Set ", vbTextCompare) = 0 Then
     
      ' see if the line is continued
      If Right(strRecord, 2) = " _" Then
          blnLineContinued = True
      End If
      
      PropertySet = True
  Else
      PropertySet = False
  End If
 
End Function

Public Function Trailer(ByVal strRecord As String) As Boolean

' ***************************************************************************
' Routine:       Trailer
'
' Description:   Parses a record to see if it is a trailer to a heading.
'                Example:    End Sub      End Function     End Property
'                            End Type     End If           Loop
'                            Wend         Next
'
' Parameters:    strRecord - data string to parse
'
' Returns:       TRUE or FALSE based on findings
'
' ===========================================================================
'    DATE      NAME / eMAIL
'              DESCRIPTION
' -----------  --------------------------------------------------------------
' 01-NOV-2000  Kenneth Ives  kenaso@home.com
'              Routine created
' ***************************************************************************

' ---------------------------------------------------------------------------
' Test the data string
' ---------------------------------------------------------------------------
  If StrComp(Left(strRecord, 12), "End Property", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 12), "End Function", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 7), "End Sub", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 6), "End If", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 4), "Loop", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 4), "Wend", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 4), "Next", vbTextCompare) = 0 Or _
     StrComp(Left(strRecord, 8), "End Type", vbTextCompare) = 0 Then
         
      Trailer = True
  Else
      Trailer = False
  End If
 
End Function

Public Function CleanUpData(strData As String) As String

' ***************************************************************************
' Routine:       CleanUpData
'
' Description:   A data string is parsed and non-printable characters are
'                replaced with nothing.
'
' Parameters:    strData - data string to be evaluated
'
' Returns:       reformatted data string
'
' ===========================================================================
'    DATE      NAME / eMAIL
'              DESCRIPTION
' -----------  --------------------------------------------------------------
' 01-NOV-2000  Kenneth Ives  kenaso@home.com
'              Routine created
' ***************************************************************************

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim intIndex As Integer
  
' ---------------------------------------------------------------------------
' Loop thru the data string and replace unwanted characters
' ---------------------------------------------------------------------------
  For intIndex = 0 To 31
      ' ignore the EOF and ESC characters
      If intIndex <> 26 Or intIndex <> 27 Then
          ' get rid of non-printable characters
          strData = Replace(strData, Chr(intIndex), vbNullString)
      End If
  Next
  
  strData = Replace(strData, Chr(34), vbNullString) ' remove all double quotes
  strData = Trim(strData)                           ' remove leading/trailing blanks
  
' ---------------------------------------------------------------------------
' Return the reformatted data
' ---------------------------------------------------------------------------
  CleanUpData = strData
  
End Function

Public Function FileExist(strFilename As String) As Boolean

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim FSO As New FileSystemObject
  
' ---------------------------------------------------------------------------
' See if the file does exist
' ---------------------------------------------------------------------------
  If Not FSO.FileExists(strFilename) Then
      MsgBox "Cannot locate:" & vbCrLf & _
      strFilename, vbInformation + vbOKOnly, _
      "File not found"
      
      FileExist = False
  Else
      FileExist = True
  End If
  
' ---------------------------------------------------------------------------
' Free the object from memory
' ---------------------------------------------------------------------------
  Set FSO = Nothing
  
End Function

Public Function FolderExist(strFolderName As String) As Boolean

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim FSO As New FileSystemObject
  
' ---------------------------------------------------------------------------
' See if the file does exist
' ---------------------------------------------------------------------------
  If Not FSO.FolderExists(strFolderName) Then
      MsgBox "Cannot locate:" & vbCrLf & _
      strFolderName, vbInformation + vbOKOnly, _
      "Folder not found"
      
      FolderExist = False
  Else
      FolderExist = True
  End If
  
' ---------------------------------------------------------------------------
' Free the object from memory
' ---------------------------------------------------------------------------
  Set FSO = Nothing
  
End Function

Public Function GetFileExtension(strFilename As String) As String

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim FSO     As New FileSystemObject
  Dim strTmp  As String
  
' ---------------------------------------------------------------------------
' See if the file does exist
' ---------------------------------------------------------------------------
  strTmp = FSO.GetExtensionName(strFilename)
      
  If Len(Trim(strTmp)) = 0 Then
      GetFileExtension = strFilename
  Else
      GetFileExtension = strTmp
  End If
  
' ---------------------------------------------------------------------------
' Free the object from memory
' ---------------------------------------------------------------------------
  Set FSO = Nothing
  
End Function

Public Function GetFileName(strPathFile As String) As String

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim FSO     As New FileSystemObject
  Dim strTmp  As String
  
' ---------------------------------------------------------------------------
' See if the file does exist
' ---------------------------------------------------------------------------
  strTmp = FSO.GetFileName(strPathFile)
  
  If Len(Trim(strTmp)) = 0 Then
      GetFileName = strPathFile
  Else
      GetFileName = strTmp
  End If
  
' ---------------------------------------------------------------------------
' Free the object from memory
' ---------------------------------------------------------------------------
  Set FSO = Nothing
  
End Function

Public Function GetPath(strPathFile As String) As String

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim FSO     As New FileSystemObject
  Dim strTmp  As String
  
' ---------------------------------------------------------------------------
' See if the file does exist
' ---------------------------------------------------------------------------
  strTmp = FSO.GetParentFolderName(strPathFile)
  
  If Len(Trim(strTmp)) = 0 Then
      GetPath = strPathFile
  Else
      GetPath = strTmp
  End If
  
' ---------------------------------------------------------------------------
' Free the object from memory
' ---------------------------------------------------------------------------
  Set FSO = Nothing
  
End Function

Public Function AddBackslash(strPath As String) As String

  If Right(strPath, 1) <> "\" Then
      AddBackslash = strPath & "\"
  Else
      AddBackslash = strPath
  End If
  
End Function

Public Function Get_Part_Of_String(ByVal strInput As String, _
                           ByVal strDelimiter As String, _
                           Optional blnFirstPart As Boolean = True) As String

' ***************************************************************************
' Routine:       Get_Part_Of_String
'
' Description:   Parses backwards thru the data string searching for a
'                particular delimter.
'
' Parameters:    strInput - Data to be parsed
'                strDelimiter - this is what we are looking for in the data
'                string
'                blnFirstPart - [Optional] Default is TRUE.  If true, return
'                all data prior to the delimiter.  If False, return all data
'                after the delimiter.  Return nothing is delimiter cannot be
'                found.
'
' Returns:       part of the data datring
'
' ===========================================================================
'    DATE      NAME / eMAIL
'              DESCRIPTION
' -----------  --------------------------------------------------------------
' 01-NOV-2000  Kenneth Ives  kenaso@home.com
'              Routine created
' ***************************************************************************

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim intPosition    As Integer
  Dim strTmp         As String
  
' ---------------------------------------------------------------------------
' Initialize variables
' ---------------------------------------------------------------------------
  strTmp = ""
  intPosition = InStrRev(strInput, strDelimiter, Len(strInput))
  
' ---------------------------------------------------------------------------
' parse the input string for the delimiter
' ---------------------------------------------------------------------------
  If blnFirstPart Then
      ' capture the first portion up to the delimiter minus one
      If intPosition > 0 Then
          strTmp = Left(strInput, intPosition - 1)
      End If
  Else
      ' capture the portion after the delimiter
      If intPosition > 0 Then
          strTmp = Mid(strInput, intPosition + 1)
      End If
  End If
  
' ---------------------------------------------------------------------------
' Return the captured portion
' ---------------------------------------------------------------------------
  Get_Part_Of_String = strTmp
  
End Function
Public Function VerifyData(strFilename As String, _
                           strPathAndFile As String) As String

' ***************************************************************************
' Routine:       VerifyPath
'
' Description:   Verify the data file exist
'
' Parameters:    strFilename - Parse this string and verify the file does
'                              exist
'                strPathAndFile - full path and file name fo the original
'                              VBP file
'
' Returns:       The full path and file
'
' ===========================================================================
'    DATE      NAME / eMAIL
'              DESCRIPTION
' -----------  --------------------------------------------------------------
' 01-NOV-2000  Kenneth Ives  kenaso@home.com
'              Routine created
' ***************************************************************************

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim strTmp   As String
  Dim strPath  As String

' ---------------------------------------------------------------------------
' Initialize variables
' ---------------------------------------------------------------------------
  strTmp = ""
  strPath = ""
  
' ---------------------------------------------------------------------------
' Get the path information
' ---------------------------------------------------------------------------
  If Left(strFilename, 2) = "\\" Or _
     Mid(strFilename, 2, 1) = ":" Or _
     Left(strFilename, 1) = "\" Then
         VerifyData = strFilename
  Else
      ' Possibility of no path information
      strPath = Get_Part_Of_String(strPathAndFile, "\", True)
      
      If Len(strPath) > 0 Then
          ' if there is data then test for folder existance
          If FolderExist(strPath) Then
              strPath = AddBackslash(strPath) ' add trailing backslash if missing
          Else
              strPath = ""
          End If
      End If
  
      strTmp = strPath & strFilename
      
      ' test for file existance
      If FileExist(strTmp) Then
          VerifyData = strTmp      ' Yes, return full path and filename
      Else
          MsgBox "Cannot locate:" & vbCrLf & _
                 strTmp, vbInformation + vbOKOnly, "File not found"
          VerifyData = ""          ' No, file does not exist
      End If
  End If

End Function

Public Function Rnd2(sngLow As Single, sngHigh As Single) As Single
  
' ***************************************************************************
' Routine:       Rnd2
'
' Description:   Create a random value between two given values.  Always get
'                into the habit of using a unique value to seed the random
'                number generator.
'
' Parameters:    sngLow  - Low end value
'                sngHign - High end value
'
' Returns:       A random generated value
'
' ===========================================================================
'    DATE      NAME / eMAIL
'              DESCRIPTION
' -----------  --------------------------------------------------------------
' 26-JUN-1999  The VB2TheMax Team  fbalena@vb2themax.com
'              Routine created
' 03-OCT-2000  Kenneth Ives  kenaso@home.com
'              Modified and documented
' ***************************************************************************
  Randomize (CDbl(Now()) + Timer)             ' always use a unique seed
  Rnd2 = (Rnd * (sngHigh - sngLow)) + sngLow

End Function

Public Function Create_Temp_Name(intLength As Integer) As String

' ***************************************************************************
' Routine:       Create_Temp_Name
'
' Description:   System generated temporary name.  User determines the length
'                of the name, not to exceed eight characters.
'
' Parameters:    intLength - length of character string desired
'
' Returns:       New name
'
' ===========================================================================
'    DATE      NAME             DESCRIPTION
' -----------  ---------------  ---------------------------------------------
' 09-APR-2000  Kenneth Ives     Modified and documented by kenaso@home.com
' ***************************************************************************
   
' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim strTmp   As String
  Dim intChar  As Integer
  
' ---------------------------------------------------------------------------
' Initialize variables
' ---------------------------------------------------------------------------
  strTmp = Chr(95)   ' First character is an underscore
  
' ---------------------------------------------------------------------------
' Test length of name desired
' ---------------------------------------------------------------------------
  Select Case intLength
         
         Case Is < 1:   ' is less than or equal zero
              Create_Temp_Name = vbNullString
              Exit Function
         Case 1:        ' Nothing to generate.  Return just an underscore.
              Create_Temp_Name = strTmp
              Exit Function
         Case Is > 7:   ' is greater than eight, reset back to eight
              intLength = 7
         Case Else:     ' default to value passed
              
  End Select
  
' ---------------------------------------------------------------------------
' Adjust new length. The first character is the underscore.  All we need is
' the rest of the string.
' ---------------------------------------------------------------------------
  intLength = intLength - 1
  
' ---------------------------------------------------------------------------
' Randomly generate characters needed (0-9, A-Z, a-z)
' ---------------------------------------------------------------------------
  Do
      intChar = Int(Rnd2(48, 122))      ' create number 65 to 122
      
      ' test the values
      Select Case intChar
             '     0 to 9    A to Z    a to z
             Case 48 To 57, 65 To 90, 97 To 122
                  ' Append new character to new name
                  ' after convert to letter of alphabet
                  strTmp = strTmp & Chr(intChar)
      End Select
      
  Loop Until Len(strTmp) = intLength
  
' ---------------------------------------------------------------------------
' Return the new name
' ---------------------------------------------------------------------------
  Create_Temp_Name = StrConv(strTmp, vbLowerCase)
  
End Function

Public Function Create_A_Temp_File(strTemp_Folder As String) As String

' ***************************************************************************
' Routine:       Create_A_Temp_File
'
' Description:   System generated temporary folder and file.  The folder
'                will be located in the Windows default temp directory and
'                is system generated.
'
'                Found this example at http://www.vbapi.com/
'
' Parameters:    strTEMP_Folder - Path to the temporary folder.
'
' Returns:       Unique name of a temporary file
'
' ===========================================================================
'    DATE      NAME             DESCRIPTION
' -----------  ---------------  ---------------------------------------------
' 02-DEC-1999  Kenneth Ives     Routine created by kenaso@home.com
' ***************************************************************************
   
' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim strPrefix   As String
  Dim strTmpFile  As String
  Dim lngRetCode  As Long
  
' ---------------------------------------------------------------------------
' Use spaces and not null values to preload the buffer.  Reduces your errors.
' ---------------------------------------------------------------------------
  strPrefix = ""
  strTmpFile = Space(MAX_SIZE)
      
' ---------------------------------------------------------------------------
' Create a unique temporary file name prefix.  Then make an API call to
' create a unique four digit hex number (Ex: 49489 becomes C151) to be
' appended to the prefix.  (example:  "_xfsC151.tmp")  The extension is
' automatically created by the API.
' ---------------------------------------------------------------------------
  strPrefix = Create_Temp_Name(3)  ' Example return:  "_xfs"
  lngRetCode = GetTempFileName(strTemp_Folder, strPrefix, 0&, strTmpFile)
  strTmpFile = Remove_Nulls(strTmpFile)
  
' ---------------------------------------------------------------------------
' Return the name of the temp file
' ---------------------------------------------------------------------------
  Create_A_Temp_File = strTmpFile
  
End Function
Public Function Get_Default_Temp_Folder() As String

' ***************************************************************************
' Routine:       Get_Default_Temp_Folder
'
' Description:   Find the windows default temporary folder
'
' Returns:       Name of the temp folder
'
' ===========================================================================
'    DATE      NAME             DESCRIPTION
' -----------  ---------------  ---------------------------------------------
' 01-NOV-2000  Kenneth Ives     Routine created by kenaso@home.com
' ***************************************************************************
   
' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim strTmpFolder  As String
  Dim lngRetCode    As Long
  
' ---------------------------------------------------------------------------
' See if we can find the Windows temporary directory path.  Use spaces and
' not null values to preload the buffer.  Reduces your errors.
' ---------------------------------------------------------------------------
  strTmpFolder = Space(MAX_SIZE)                     ' initialize buffer
  lngRetCode = GetTempPath(MAX_SIZE, strTmpFolder) ' read the path name
  
' ---------------------------------------------------------------------------
' extract data from the variable (ex:  "C:\Temp\")
' ---------------------------------------------------------------------------
  If lngRetCode Then
      ' We found the Windows default Temp folder.  Remove
      ' the trialing nulls and append a backslash, if missing
      strTmpFolder = Remove_Nulls(strTmpFolder)
      strTmpFolder = AddBackslash(strTmpFolder)
  Else
      strTmpFolder = "C:\" ' default to the root directory of drive C:
  End If
  
' ---------------------------------------------------------------------------
' Return the temp folder location
' ---------------------------------------------------------------------------
  Get_Default_Temp_Folder = strTmpFolder
  
End Function

Public Function Remove_Nulls(strInput As String) As String
    
' ***************************************************************************
' Routine:       Remove_Nulls
'
' Description:   Receives a data string and looks for the first null.  If
'                found, the string is truncated at that point and returned.
'
' Parameters:    strInput - Input string to be inspected
'
' Returns:       Data string with the trailing nulls
'
' ===========================================================================
'    DATE      NAME             DESCRIPTION
' -----------  ---------------  ---------------------------------------------
' 23-FEB-2000  Kenneth Ives     Routine created by kenaso@home.com
' ***************************************************************************

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim intPosition As Integer
    
' ---------------------------------------------------------------------------
' Look for first null in the data string
' ---------------------------------------------------------------------------
  intPosition = InStr(1, strInput, Chr(0))
  
' ---------------------------------------------------------------------------
' if we found one, then save all the data up to that position
' ---------------------------------------------------------------------------
  If intPosition Then
      Remove_Nulls = Left(strInput, intPosition - 1)
  Else
      Remove_Nulls = strInput     ' no nulls found
  End If
      
End Function

Public Function Shrink_2_Fit(ByVal strPath_To_Resize As String, _
                             intMaxLen As Integer) As String

' ***************************************************************************
' Routine:       Shrink_2_Fit
'
' Description:   This function will shorten a directory name to the length
'                passed to the Max parameter.
'
' Syntax:
'        Shrink_2_Fit "C:\Program Files\Navigator\Programs\Bookmark.htm", 30
'        Returns -->  "C:\...\Programs\Bookmark.htm"
'
' Parameters:    strPath_to_Resize - Path to be resized for display
'                intMaxLen - Maximum length of the return string
'
' Returns:       Resized path
'
' ===========================================================================
'    DATE      NAME             DESCRIPTION
' -----------  ---------------  ---------------------------------------------
' 21-FEB-1998  Kenneth Ives     Written by kenaso@home.com
' ***************************************************************************

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim intPosition  As Integer
  Dim intStart     As Integer
  Dim intNeeded    As Integer
  Dim intLength    As Integer
  Dim strTmp       As String
  Dim strPrefix    As String
  
' ---------------------------------------------------------------------------
' Initialize variables
' ---------------------------------------------------------------------------
  strTmp = Trim(strPath_To_Resize)

' ---------------------------------------------------------------------------
' if the string is equal to or less than the desired length then leave
' this routine
' ---------------------------------------------------------------------------
  If Len(strTmp) <= intMaxLen Then
      Shrink_2_Fit = strTmp
      Exit Function
  End If

' ---------------------------------------------------------------------------
' Prepare the drive and the elipces.  Start in the third position in case
' this is a UNC (Universal Naming Convention) path
' ---------------------------------------------------------------------------
  If Left(strTmp, 2) = "\\" Or Mid(strTmp, 2, 1) = ":" Then
      intPosition = InStr(3, strTmp, "\")            ' search for next backslash
      intStart = intPosition + 1                     ' calc new starting position
      intNeeded = intMaxLen - (intPosition + 3)      ' Add for 3 elipces
  Else
      Shrink_2_Fit = strTmp   ' do not know what this is.  get rid of it.
      Exit Function
  End If
  
' ---------------------------------------------------------------------------
' Build the prefix.  A drive letter or server_name, backslash and 3 elipces
' Example:   "C:\..." or "\\Server\..."
' ---------------------------------------------------------------------------
  strPrefix = Left(strTmp, intPosition) & "..."
  
' ---------------------------------------------------------------------------
' prepare the suffix part of the path to the desired length
' ---------------------------------------------------------------------------
  Do
      intPosition = InStr(intStart, strTmp, "\") ' look for a backslash
      
      If intPosition > 0 Then
          strTmp = Mid(strTmp, intPosition)      ' Reformat string length
          intLength = Len(strTmp)
      Else
          Exit Do                                ' could not find "\", so leave
      End If
      
  Loop Until intNeeded >= intLength
  
' ---------------------------------------------------------------------------
' Return the readjusted data string
' ---------------------------------------------------------------------------
  Shrink_2_Fit = strPrefix & strTmp
  
End Function

Public Sub QuickSort_String_Ascending(arString() As String, _
                                      lngLow As Long, lngHi As Long)

' ***************************************************************************
' Routine:       QuickSort_String_Ascending
'
' Description:   This routine will accept and sort in ascending order a
'                string array of data.  This routine is used when the data
'                to be sorted is known to be ALL string.
'
' Parameters:    arString() - Array to be sorted
'                lngLow     - Minimum number of elements in the array
'                lngHi      - Maximum numer of elements in the array
'
' ===========================================================================
'    DATE      NAME             DESCRIPTION
' -----------  ---------------  ---------------------------------------------
' 30-APR-2000  Kenneth Ives     Routine created kenaso@home.com
' ***************************************************************************

' ----------------------------------------------------------------------
' Define local striables
' ----------------------------------------------------------------------
  Dim strMidPoint  As String    ' midpoint of the array to be sorted
  Dim strHold      As String    ' Temp hold area for swapping values
  Dim lngTmpLow    As Long      ' Index pointer
  Dim lngTmpHi     As Long      ' Index pointer
   
' ----------------------------------------------------------------------
' See if this is an empty array by checking to see if there is data in
' the first element.  If not, then leave.
' ----------------------------------------------------------------------
  If Len(Trim(arString(0))) = 0 Then
      Exit Sub
  End If
  
' ----------------------------------------------------------------------
' Leave if there is nothing to sort
' ----------------------------------------------------------------------
  If lngLow >= lngHi Then
      Exit Sub
  End If

' ----------------------------------------------------------------------
' Save the count of the minimum and maximum number of elements in the
' array to be sorted.
' ----------------------------------------------------------------------
  lngTmpLow = lngLow
  lngTmpHi = lngHi
   
' ----------------------------------------------------------------------
' Calculate the midpoint of the array
' ----------------------------------------------------------------------
  strMidPoint = arString((lngLow + lngHi) / 2)

' ----------------------------------------------------------------------
' Start the sorting process
' ----------------------------------------------------------------------
  While (lngTmpLow <= lngTmpHi)
       
      ' Always process the low end first.  Loop as long the array data
      ' element is LESS than the data in the temporary holding area
      ' and the temporary low value is LESS than the maximum number of
      ' array elements.  To make an accurate sort for string data, we
      ' temporarily convert the data to Uppercase to make the comparisons.
      ' This is because Uppercase will collate before Lowercase.
      While (StrConv(arString(lngTmpLow), vbUpperCase) < StrConv(strMidPoint, vbUpperCase) And lngTmpLow < lngHi)
          lngTmpLow = lngTmpLow + 1  ' Increment the temp low counter
      Wend
   
      ' Now, we will process the high end.  Loop as long the data in the
      ' temporary holding area is LESS than the array data element
      ' and the temporary high value is GREATER than the minimum number
      ' of array elements.  To make an accurate sort for string data, we
      ' temporarily convert the data to Uppercase to make the comparisons.
      ' This is because Uppercase will collate before Lowercase.
      While (StrConv(strMidPoint, vbUpperCase) < StrConv(arString(lngTmpHi), vbUpperCase) And lngTmpHi > lngLow)
          lngTmpHi = lngTmpHi - 1    ' Decrement the temp high counter
      Wend

      ' if the temp low end is LESS than or equal to the temp high end,
      ' then swap places
      If (lngTmpLow <= lngTmpHi) Then
          Swap_Data arString(lngTmpLow), arString(lngTmpHi)
          lngTmpLow = lngTmpLow + 1   ' Increment the temp low counter
          lngTmpHi = lngTmpHi - 1     ' Decrement the temp high counter
      End If
 Wend
    
' ----------------------------------------------------------------------
' If the minimum number of elements in the array is LESS than the temp
' high end, then make a recursive call to this routine.  Always sort
' the low end of the array first.  This gives you a solid base.
' ----------------------------------------------------------------------
  If (lngLow < lngTmpHi) Then
      QuickSort_String_Ascending arString(), lngLow, lngTmpHi
  End If
   
' ----------------------------------------------------------------------
' If the temp low end is LESS than the maximum number of elements in
' the array, then make a recursive call to this routine.  The high end
' is always sorted last.
' ----------------------------------------------------------------------
  If (lngTmpLow < lngHi) Then
      QuickSort_String_Ascending arString(), lngTmpLow, lngHi
  End If

End Sub

Private Function Swap_Data(strData1 As String, strData2 As String)

' ***************************************************************************
' Routine:       Swap_Data
'
' Description:   Swap data with each other.  I wrote this function since
'                BASIC stopped having its own SWAP function.
'
' Parameters:    strData1 - string integer to be swapped with strData2
'                strData2 - string integer to be swapped with strData1
'
' Return Values: Swapped data
'
' ===========================================================================
'    DATE      NAME             DESCRIPTION
' -----------  ---------------  ---------------------------------------------
' 15-JUN-2000  Kenneth Ives     Routine created by kenaso@home.com
' ***************************************************************************
    
' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim strTmp As String
  
' ---------------------------------------------------------------------------
' Swap the values with each other
' ---------------------------------------------------------------------------
  strTmp = strData1
  strData1 = strData2
  strData2 = strTmp
  
End Function


