VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsMiscForm"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Attribute VB_Description = "Contains miscellaneous functions pertaining to forms."
Attribute VB_Ext_KEY = "SavedWithClassBuilder6" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
Option Explicit

' ---------------------------------------------------------------------------
' Adapted from an example in VBPJ Tech Tips, 10th Edition.
'
' Declares needed for centering a form on the screen
' ---------------------------------------------------------------------------
  Private Const SM_CXFULLSCREEN  As Long = 16
  Private Const SM_CYFULLSCREEN  As Long = 17

  Private Declare Function GetSystemMetrics Lib "user32" _
          (ByVal nIndex As Long) As Long

  Private Declare Function GetWindowLong Lib "user32" _
          Alias "GetWindowLongA" (ByVal hwnd As Long, _
          ByVal nIndex As Long) As Long

' ---------------------------------------------------------------------------
' Define constants
' ---------------------------------------------------------------------------
  Private Const SW_SHOWNORMAL           As Long = 1
  Private Const SW_SHOWMINIMIZED        As Long = 2
  Private Const SW_SHOWNOACTIVATE       As Long = 4
  Private Const SW_RESTORE              As Long = 9
  Private Const SWP_NOACTIVATE          As Long = &H10
  Private Const SWP_NOZORDER            As Long = &H4
  Private Const SWP_SHOWWINDOW          As Long = &H40
  Private Const SWP_HIDEWINDOW          As Long = &H80
  Private Const SWP_NOMOVE              As Long = 2
  Private Const SWP_NOSIZE              As Long = 1
  Private Const HWND_FLAGS              As Long = SWP_NOMOVE Or SWP_NOSIZE
  Private Const HWND_TOPMOST            As Long = -1
  Private Const HWND_NOTOPMOST          As Long = -2
  Private Const WM_CLOSE                As Long = &H10
  Private Const WM_USER                 As Long = &H400
  Private Const SRCCOPY                 As Long = &HCC0020
  Private Const SRCAND                  As Long = &H8800C6
  Private Const SRCINVERT               As Long = &H660046
  Private Const SPI_SCREENSAVERRUNNING  As Long = 97&
  

  Private Type RECT
       Left    As Long
       Top     As Long
       Right   As Long
       Bottom  As Long
  End Type

  Private Type POINTAPI
       X       As Long
       Y       As Long
  End Type

  Private Type WINDOWPLACEMENT
       Length            As Long
       flags             As Long
       showCmd           As Long
       ptMinPosition     As POINTAPI
       ptMaxPosition     As POINTAPI
       rcNormalPosition  As RECT
  End Type

' ------------------------------------------------------------------
'               Form Always on Top
' ------------------------------------------------------------------
  Private Declare Function SetWindowPos Lib "user32" _
          (ByVal hwnd As Long, ByVal hWndInsertAfter As Long, _
          ByVal X As Long, ByVal Y As Long, ByVal cx As Long, _
          ByVal cy As Long, ByVal wFlags As Long) As Long
                    
  Private Declare Function SystemParametersInfo Lib "user32" _
          Alias "SystemParametersInfoA" (ByVal uAction As Long, _
          ByVal uParam As Long, ByVal lpvParam As Any, _
          ByVal fuWinIni As Long) As Long
          
' ---------------------------------------------------------------------------
' Declares required for verifying a previous instance of program executiion
' ---------------------------------------------------------------------------
  Private Declare Function FindWindow Lib "user32" _
          Alias "FindWindowA" (ByVal lpClassName As String, _
          ByVal lpWindowName As String) As Long
          
  Private Declare Function SetWindowPlacement Lib "user32" _
          (ByVal hwnd As Long, lpwndpl As WINDOWPLACEMENT) As Long
  
  Private Declare Function SetForegroundWindow Lib "user32" _
          (ByVal hwnd As Long) As Long
  
  Private Declare Function GetWindowPlacement Lib "user32" _
          (ByVal hwnd As Long, lpwndpl As WINDOWPLACEMENT) As Long
  
  Private Declare Function SendMessageArray Lib "user32" _
          Alias "SendMessageA" (ByVal hwnd As Long, ByVal wMsg As Long, _
          ByVal wParam As Long, lParam As Any) As Long

' ---------------------------------------------------------------------------
' Remove the "X" from the window and menu
' ---------------------------------------------------------------------------
  Private Const MF_BYPOSITION   As Long = &H400
  Private Const MF_REMOVE       As Long = &H1000

  Private Declare Function DrawMenuBar Lib "user32" _
          (ByVal hwnd As Long) As Long
  
  Private Declare Function GetMenuItemCount Lib "user32" _
          (ByVal hMenu As Long) As Long
  
  Private Declare Function GetSystemMenu Lib "user32" _
          (ByVal hwnd As Long, ByVal bRevert As Long) As Long
  
  Private Declare Function RemoveMenu Lib "user32" _
          (ByVal hMenu As Long, ByVal nPosition As Long, _
          ByVal wFlags As Long) As Long

' ---------------------------------------------------------------------------
' This is a rough translation of the GetTickCount API. The tickcount of a
' PC is only valid for the first 49.7 days since it was last rebooted.
' When you capture the tickcount, you are capturing the total number of
' milliseconds since the PC was last rebooted.
' ---------------------------------------------------------------------------
  Private Declare Function GetTickCount Lib "kernel32" () As Long
  
' ---------------------------------------------------------------------------
' Module level variables (Prefixed by "m_")
' ---------------------------------------------------------------------------
  Private m_lngRetVal   As Long

Public Sub Delay(intAmtOfDelay As Integer, _
                 Optional strTypeOfDelay As String = "s")
Attribute Delay.VB_Description = "Pauses an application based on length of time and if it is to be in 's'-Seocnds, 'n'-Minutes, 'h'-Hours, etc."

' ---------------------------------------------------------------------------
' This routine will cause a delay for the time requested, yet will not
' interfere with the program progress like the Sleep API.  This routine
' does not rely on the Timer event, a timer control, or the tickcount of
' the last reboot.  All of these have their drawbacks when dealing with time
' comparisons.
'
' Timer control is only valid from midnight of that particular day.  If you
' want to delay 5 minutes and the time is 23:59:00.  You will never reach
' the finish time because at midnight, the timer control is reset.
'
' The timer event is based on a single precision caluclation of the date and
' time.  to the left of the decimal is the date and to the right is the time.
' Somewhere in the 24 hour cycle, this is reset.  I suspect midnight.  this
' is good for immediate testing but not for comparisons.
'
' I use the system date and time by calling the VB function Now().  As long
' as the machine is running, it will have a system date and time stamp that
' is being updated.
'
' Parameters:
'       intAmtOfDelay - amount of time to delay
' ---------------------------------------------------------------------------

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim varDelayTime  As Variant
  Dim varCurrTime   As Variant
  
' ---------------------------------------------------------------------------
' Determine the length of time to delay using the VB DateAdd function.
'
'    "s" - seconds      "d" - days
'    "n" - minutes      "m" - months
'    "h" - hours        "yyyy" - years
'
' We are adding the amount of delay to the current time
' ---------------------------------------------------------------------------
  varDelayTime = Format(DateAdd(strTypeOfDelay, intAmtOfDelay, Now), "hh:mm:ss")
  
' ---------------------------------------------------------------------------
' Loop thru and continualy check the curent time with the calculated time so
' we know when to leave
' ---------------------------------------------------------------------------
  Do
      varCurrTime = Format(Now, "hh:mm:ss")
      DoEvents
      DoEvents
      
      ' if the string1 is greater than string2,
      ' a one will be returned
      If StrComp(varCurrTime, varDelayTime) = 1 Then
          Exit Do
      End If
  Loop

End Sub

Public Sub Mini_Delay(lngDelayAmt As Long)
Attribute Mini_Delay.VB_Description = "Will delay an application based on milliseconds."

' ---------------------------------------------------------------------------
' This routine will cause a delay for the time requested, yet will not
' interfere with the program progress like the Sleep API.  I use this for
' other than whole seconds.
'
' Remenber 1000 milliseconds = 1 second.
'
' Parameters:
'       lngDelayAmt - amount of time to delay
'                   Tickcount + x number of milliseconds
'
' syntax:     Mini_Delay 750     ' 3/4 of a second
' ---------------------------------------------------------------------------

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim lngNewTime   As Long
  Dim lngCurrent   As Long
  
' ---------------------------------------------------------------------------
' Calculate the new waiting time
' ---------------------------------------------------------------------------
  lngNewTime = GetTickCount + lngDelayAmt
  
' ---------------------------------------------------------------------------
' Loop thru and continualy check the curent time with the calculated time so
' we know when to leave
' ---------------------------------------------------------------------------
  Do
      lngCurrent = GetTickCount      ' get the current millisecond count
      DoEvents
      
      ' if the current millisecond count has not
      ' caught up with the delay amount then
      ' we will try again.
      If lngCurrent >= lngNewTime Then
          Exit Do
      End If
  Loop

End Sub

Public Sub RemoveX(frm As Form)
Attribute RemoveX.VB_Description = "Disables the 'X' in the upper right corner of a form and the 'Close' in the control box in the upper left corner."

' ---------------------------------------------------------------------------
' For completeness, you may want to confirm that the menuItemCount
' matches the value you expect before performing the removal. For
' example, on a normal form with a full system menu, menuItemCount will
' return seven.
'
' But what if your application is an MDI app and you want to disable the
' close button on the parent?  Just pass MDIForm1.hwnd as the form hwnd
' parameter in the GetSystemMenu() call.
' ---------------------------------------------------------------------------
   
' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim hMenu           As Long
  Dim menuItemCount   As Long

' ---------------------------------------------------------------------------
' Obtain the handle to the form's system menu
' ---------------------------------------------------------------------------
  hMenu = GetSystemMenu(frm.hwnd, 0)
  
  If hMenu Then
      
     ' Obtain the number of items in the menu
      menuItemCount = GetMenuItemCount(hMenu)
    
     ' Remove the system menu Close menu item.
     ' The menu item is 0-based, so the last
     ' item on the menu is menuItemCount - 1
      Call RemoveMenu(hMenu, menuItemCount - 1, MF_REMOVE Or MF_BYPOSITION)
   
     ' Remove the system menu separator line
      Call RemoveMenu(hMenu, menuItemCount - 2, MF_REMOVE Or MF_BYPOSITION)
    
     ' Force a redraw of the menu. This
     ' refreshes the titlebar, dimming the X
      Call DrawMenuBar(frm.hwnd)

   End If
   
End Sub

Public Function IsAnotherInstance(strApplName As String) As Boolean
Attribute IsAnotherInstance.VB_Description = "Test for the existance of another instance of an application being executed."

' ---------------------------------------------------------------------------
' Call this module from the Sub Main() or Form_Load() event.
' ---------------------------------------------------------------------------

' ---------------------------------------------------------------------------
' Define local variable
' ---------------------------------------------------------------------------
  Dim strSaveTitle As String
  
' ---------------------------------------------------------------------------
' Check for a previous instance of a program running
' ---------------------------------------------------------------------------
  If App.PrevInstance Then

      ' change the new instance title to prevent it
      ' from being located instead of the original
      ' instance.  Note however that as this is in
      ' a BAS module and not the form load sub,
      ' change "pgm_name" to the name of the application
      ' that you do not a dupliate instance of.
      strSaveTitle = App.Title
      App.Title = strApplName     ' name of executable w/o .exe extention
      RestorePreviousInstance strSaveTitle
      IsAnotherInstance = True
      Exit Function
  End If

  IsAnotherInstance = False
  
End Function

Private Sub RestorePreviousInstance(strPreviousTitle As String)
Attribute RestorePreviousInstance.VB_Description = "Restores the previous instance of an application."

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim lngHandle    As Long
  Dim WinLocation  As WINDOWPLACEMENT
     
' ---------------------------------------------------------------------------
' VB3 & VB4 use class name "ThunderRTForm"
' VB5 uses class name "ThunderRT5Form"
' VB6 uses class name "ThunderRT6FormDC"
'
' Including the class name for the compiled EXE class prevents the routine
' from finding and attempting to activate the project form of the same name.
' ---------------------------------------------------------------------------
  lngHandle = FindWindow("ThunderRT6FormDC", strPreviousTitle)
  DoEvents
   
' ---------------------------------------------------------------------------
' If application is already executing
' ---------------------------------------------------------------------------
  If lngHandle > 0 Then
      ' get the current window state of the previous instance
      WinLocation.Length = Len(WinLocation)
      m_lngRetVal = GetWindowPlacement(lngHandle, WinLocation)
               
      ' if the WinLocation.showCmd member indicates that
      ' the window is currently minimized, it needs
      ' to be restored.
      If WinLocation.showCmd = SW_SHOWMINIMIZED Then
          With WinLocation
               .Length = Len(WinLocation)
               .flags = 0&
               .showCmd = SW_SHOWNORMAL
          End With
          m_lngRetVal = SetWindowPlacement(lngHandle, WinLocation)
      End If
       
      ' Bring the window to the front and make the active window.
      ' Without this, it may remain hidden behind other windows.
      m_lngRetVal = SetForegroundWindow(lngHandle)
      DoEvents
  End If
   
End Sub

Public Function CenterForm(objForm As Form) As Boolean
Attribute CenterForm.VB_Description = "Centers a form on the screen taking into account any tool or task bars present."

' ---------------------------------------------------------------------------
' Adapted from an example in VBPJ Tech Tips, 10th Edition.
'
' Centers form on screen, accounting for taskbars such as the Windows
' taskbar, MS Office taskbar, etc.  Pass the form you want to center as
' the objForm parameter.  Returns TRUE if successful, FALSE if not.
'
' Scale mode of form must be vbTwips, otherwise this won't work.
'
' if you think that form's scale mode might be something else, incorporate
' the perpixelx and perpixely functions for metrics other than twips.
' ---------------------------------------------------------------------------

' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim lngLeft  As Long
  Dim lngTop   As Long
        
  On Error GoTo Normal_Exit
' ---------------------------------------------------------------------------
' If the scale mode is not in TWIPS then leave
' ---------------------------------------------------------------------------
  If objForm.ScaleMode <> vbTwips Then
      CenterForm = False
      Exit Function
  End If

' ---------------------------------------------------------------------------
' Determine the full width of the screen.  Then calculate the width
' of the actual usable surface. Then calculate the centering of the form
' from left to right.
' ---------------------------------------------------------------------------
  lngLeft = (Screen.TwipsPerPixelX * GetSystemMetrics(SM_CXFULLSCREEN)) / 2
  lngLeft = lngLeft - (objForm.Width / 2)
        
' ---------------------------------------------------------------------------
' Determine the full height of the screen.  Then calculate the height
' of the actual usable surface.  Then calculate the centering of the form
' from top to bottom.
' ---------------------------------------------------------------------------
  lngTop = (Screen.TwipsPerPixelY * GetSystemMetrics(SM_CYFULLSCREEN)) / 2
  lngTop = lngTop - (objForm.Height / 2)
        
' ---------------------------------------------------------------------------
' Now move the form into position
' ---------------------------------------------------------------------------
  objForm.Move lngLeft, lngTop
        
Normal_Exit:
' ---------------------------------------------------------------------------
' If not error occured then return a TRUE else FALSE
' ---------------------------------------------------------------------------
  CenterForm = (Err.Number And Err.LastDllError)

End Function

Public Sub AlwaysOnTop(frm As Form, blnOnTop As Boolean)

' ---------------------------------------------------------------------------
' This routine uses an argument to determine whether to make the specified
' form always on top or not.
'
' Syntax:  AlwaysOnTop form_name, True    ' Place on top
' ---------------------------------------------------------------------------
  If blnOnTop Then
      m_lngRetVal = SetWindowPos(frm.hwnd, HWND_TOPMOST, 0&, 0&, 0&, 0&, HWND_FLAGS)
  Else
      m_lngRetVal = SetWindowPos(frm.hwnd, HWND_NOTOPMOST, 0&, 0&, 0&, 0&, HWND_FLAGS)
  End If

End Sub

Public Sub CenterCaption(frm As Form, strText As String)

' ---------------------------------------------------------------------------
' This Sub will automatically center the forms caption on startup.
' ---------------------------------------------------------------------------
  
' ---------------------------------------------------------------------------
' Define local variables
' ---------------------------------------------------------------------------
  Dim lngTextWidth  As Long
  Dim lngFormWidth  As Long
  Dim lngHalfForm   As Long
  
' ---------------------------------------------------------------------------
' See if anything was passed to here
' ---------------------------------------------------------------------------
  If Len(Trim(strText)) = 0 Then
      Exit Sub
  End If

' ---------------------------------------------------------------------------
' Center the text on the caption line
' ---------------------------------------------------------------------------
  With frm
       .ScaleMode = vbCharacters          ' Sets the formscale To characters
       lngTextWidth = .TextWidth(strText) ' Gets the string length
       lngFormWidth = .ScaleWidth         ' Gets the forms width
       ' Halfs the form for the center
       lngHalfForm = lngFormWidth / 2
       .Caption = Space(lngHalfForm) & strText ' Sets the forms caption
       .ScaleMode = vbTwips               ' Sets the formscale To twips again
  End With
  
End Sub
